//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                         SpikenzieLabs.com                                                     *
//  *                                                                                                               *
//  *                                     VoiceShield Programmer Lite                                               *
//  *                                                                                                               *
//  *                             Program to load Audio into the VoiceShield                                        *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
// BY: MARK DEMERS 
// Jan. 2009
// VERSION: 1.1
//
// DESCRIPTION:
// This is program to load Audio into the VoiceShield from Spikenzielabs.com.
//
// The default serial configuration is 9600 8N1
//
// FUNCTION:
// A. Collect Audio sound bytes to load into the Voice Shield, place in folder "SoundBytes" with this App.
// B. Save your playlist into a TAB separated text file, [slot number TAB sound name with extension] .
// C. Connect the Arduino with VoiceShield and computer. (Arduino must be preloaded with the VSloader Sketch)
// D. Run this software - User selects a serial port form a list.
// E. Click on "Program", wait for your Voice Shield to be loaded.
// 
// PROGRAM:
// Coded in Processing 1.0.1 and uses the minim v 1.1 library (now part of Processing).
//
// LEGAL:
// This code is provided as is. No guaranties or warranties are given in any form. It is your responsibilty to 
// determine this codes suitability for your application.

import ddf.minim.*;
import processing.serial.*;

Minim minim;
AudioSnippet song;

Serial port;                                                                    // Create object from Serial class
//AudioPlayer song;
Button b1, b2, b3, b4;

String[] lines;
int index = 0;
int state =3;
int i = 0;
int sampletime = 0;
String ports[] =  Serial.list();
String StatusText =  "Not selected";
String MainTextLineOne = "";
String MainTextLineTwo = "";
int SPortCount = 0;
float SampTime = 0;
boolean SampPlayFlag = false;
boolean SerPortFlag =  false;
boolean SongOpenFlag = false;

void setup()
{
  size(380, 500);
  textFont(createFont("Arial", 12));
  frameRate(25);
  background(255);

 // Minim.start(this);                                                               // always start Minim first!
  minim = new Minim(this);

  lines = loadStrings("SoundScore.txt");

  b1 = new Button(15,40,72,24,"Preview",1);                                        // Preview
  b2 = new Button(105,40,72,24,"Program",2);                                       // Program
  b3 = new Button(195,40,72,24,"Port",3);                                          // Select Port
  b4 = new Button(285,40,72,24,"Stop",4);                                          // Select Port

/*
//  song = Minim.loadFile("/SoundBytes/"+"Monkey2.aiff");                            // Play any audio to get the Minim started
    song = minim.loadSnippet("/SoundBytes/"+"Monkey2.aiff");
    SongOpenFlag = true;
    song.play();

    delay(song.length());
    song.close();
    SongOpenFlag = false;
*/
}

void draw()
{  
  switch(state)
  {
  case 0:                                                                           // Waiting State
    MainTextLineOne = "";
    updatedisplay();
    break;

  case 1:                                                                           // Preview State 
    StatusText = "PREVIEW MODE";
    updatedisplay();
    if (index < lines.length)
    {
      if(SampPlayFlag == false)
      {
        String[] pieces = split(lines[index], '\t');      
        MainTextLineOne = ("SLOT: " + "     "+ " FILE: ");
        MainTextLineTwo = (pieces[0]   + "                 "+ pieces[1]);
        String AudFile = pieces[1];

        SampPlayFlag = true;
        ISD(AudFile);
        SampTime = millis();
        SampTime = SampTime + sampletime;
      }
      else
      {
        if(millis() > SampTime)
        {
          index = index + 1;
          SampPlayFlag = false;
          MainTextLineOne = "";
          song.close();
          SongOpenFlag = false;
        }
      }
    }
    else
    {
      index = 0;
      state = 0;
      MainTextLineOne = "";
      MainTextLineTwo = "";
      StatusText = "Preview Finished";
    }
    break;

  case 2:                                                                               // Program VS State 
    StatusText = "RECORDING";
    updatedisplay();
    if(SerPortFlag == true)
    {
      if (index < lines.length)
      {
        if(SampPlayFlag == false)
        {
          String[] pieces = split(lines[index], '\t');      
   //       MainTextLineOne = ("RECORD: " + pieces[1] + " TO SOUND SLOT: "+ pieces[0]);
          MainTextLineOne = ("SLOT: " + "     "+ " FILE: ");
          MainTextLineTwo = (pieces[0]   + "                 "+ pieces[1]);
          String AudFile = pieces[1];

          port.write(255);                                                                // Start BYTE
          port.write(3);                                                                  // RECORD COMMAND
          port.write(int(pieces[0]));                                                     // The ADDRESS   

          delay(950);                         // Rac time + RAC Time + Racl time 
          // N.B You may adjust this value, but without a long enough delay the start of your Audio sample may get cliped 
          // since the ISD is "slow" to start recording.

          SampPlayFlag = true;
          ISD(AudFile);
          SampTime = millis();
          SampTime = SampTime + sampletime;
        }
        else
        {
          if(millis() > SampTime)
          {
            index = index + 1;
            SampPlayFlag = false;
            MainTextLineOne = "";

            port.write(255);                                                             // Start BYTE
            port.write(1);                                                               // STOP COMMAND
            port.write(0);                                                               // not a real ADDRESS (STOP does not use an Address)

            song.close();
            SongOpenFlag = false;
          }
        }
      }
      else
      {
        index = 0;
        state = 0;
        MainTextLineOne = "";
        MainTextLineTwo = "";
        StatusText = "Finished Recording";
      }
    }
    else
    {
      StatusText = "ERROR: OPEN SERIAL PORT FIRST";
      index = 0;
      state = 0;
      MainTextLineOne = "";
      MainTextLineTwo = "";
    }
    break;

  case 3:                                                                                 // Select Serial Port State 
    updatedisplay();
    selectport();
    break;
    
  case 4:                                                                                 // STOP 
    MainTextLineOne = "";
    MainTextLineTwo = "";
    StatusText = "Stopped";
    SampPlayFlag = false;
    state= 0;
    index = 0;
    if(SerPortFlag == true)
    {
      port.write(255);                                                                    // Start BYTE
      port.write(1);                                                                      // STOP COMMAND
      port.write(0);                                                                      // not a real ADDRESS (STOP does not use an Address)
    }  
  /*  
    if(SongOpenFlag == true)
    {
    song.close();
    }
  */
    
    updatedisplay();
    break;  
  }  
}

void updatedisplay()
{
  background(255);
  int fadecolor = 220;

  for (int i = 0; i < 80; i = i+1)                                                       // Top area
  {
    stroke(fadecolor);
    line(0, i, width, i);
    fadecolor = fadecolor -2;
  }
  fadecolor=128;
  for (int i = height-40; i < height; i = i+1)                                           // Bottom area
  {
    stroke(fadecolor);
    line(0, i, width, i);
    fadecolor = fadecolor + 3;
  }

  fadecolor=50;
  for (int i = height-60; i < height-40; i = i+1)                                         // Upper-Bottom area
  {
    stroke(fadecolor);
    line(0, i, width, i);
    fadecolor = fadecolor + 3;
  }
  
  fill(255);                                                                             // Status Title
  stroke(0);
  text("Status:", 15, height - 45);


  fill(0);                                                                                // Status Text
  stroke(50);
  text(StatusText, 15, height-25, 600, 350);

  if(state != 3)
  {
    text(MainTextLineOne, 15, 100);
    text(MainTextLineTwo, 15, 120);
  }

  b1.display();
  b2.display();
  b3.display();
  b4.display();

  textAlign(CENTER);
  fill(255);
  textSize(18);
//  text("VoiceShield Programmer Lite v1.0", (width/2)-(300/2),10,300,20);
  text("VoiceShield Programmer Lite v1.0", (width/2),25);
  textAlign(LEFT);
  textSize(12);
}

void ISD(String PlayFile)
{
//  song = Minim.loadFile("/SoundBytes/"+PlayFile);
  song = minim.loadSnippet("/SoundBytes/"+PlayFile);
  SongOpenFlag = true;
  song.play();
  //  delay(song.length());
  sampletime = (song.length())+100;
}


void keyPressed() {
  if(state==3){
    if((key-48) <=  (Serial.list().length-1))
    {
      state=0;
      port = new Serial(this,ports[key-48], 9600);
      StatusText = ("You are using serial port ["+(key-48)+"] : "+ports[key-48]);
      SerPortFlag = true;
    }
  }
}

void mouseReleased(){
  switch(state) {
  case 0: 
    break;
  case 1:                                  // Preview
    if(b1.bover()){
    }
  case 2:                                  // Program
    if(b2.bover()){

    }
  case 3:                                  // Select Port
    if(b3.bover()){
      selectport();
    }
  case 4:                                  // Select Port
    if(b3.bover()){
      selectport();
    }
  }
}


class Button 
{
  int bxpos; 
  int bypos;
  int wid;
  int hig;
  String name;
  int  Bstate;
  int updownval;

  Button(int ixp, int iyp, int iw, int ih, String iname, int istate) {
    bxpos = ixp;
    bypos = iyp;
    wid = iw;
    hig = ih;
    name = iname;
    Bstate = istate;
  }

  void display() {
    if(bover() && mousePressed == true) {
      fill(128);
      stroke(50);
      rect(bxpos,bypos,wid,hig);
      state = Bstate;
    } 
    else if (bover())
    {
      fill(128);
      stroke(50);
      rect(bxpos,bypos,wid,hig);
    }
    else {
      fill(50);
      stroke(50);
      rect(bxpos,bypos,wid,hig);
    }

    textAlign(CENTER);
    fill(255);
    stroke(255);

    text(name, bxpos,bypos+5,wid,hig);
    textAlign(LEFT);
  }

  boolean bover() {
    if(mouseX >= bxpos && mouseX <= bxpos+wid && 
      mouseY >= bypos && mouseY <= bypos+hig) {
      return true;
    } 
    else {
      return false;
    }
  }
}

void selectport() 
{
  StatusText = "Press the number key that matches your Arduino's serial Port:";
  SPortCount = Serial.list().length;
  if(SPortCount==0)
  {
    StatusText = "ERROR: No serial Ports avaliable";
  }
  else
  {
    fill(0);
    for(i=0; i <Serial.list().length; i=i+1)
    {
      text("Port ["+i+"] :    "+ports[i], 15, 100+(i*25), 600, 350);
    }
  }
  if(SerPortFlag)
  {
    port.stop();
    SerPortFlag = false;
  }
}

void stop()
{
 // song.close();
  minim.stop();
  super.stop();
}
